/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.tm.core.BentenTmEngine;
import benten.cat.tm.core.BentenTmSearchResult;
import benten.core.text.Strings;
import benten.ui.AbstractBentenUiPlugin;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * The activator class controls the plug-in life cycle
 */
public class CatTmUiPlugin extends AbstractBentenUiPlugin {

	// The shared instance
	private static CatTmUiPlugin plugin;

	private BentenTmEngine tm = null;

	/**
	 * The constructor
	 */
	public CatTmUiPlugin() {
	}

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static CatTmUiPlugin getDefault() {
		return plugin;
	}

	public void loadExtensions() {
		final BentenTmEngine tm = newTmEngine();

		if (tm != null) {
			tm.load();
			this.tm = tm;
		}
	}

	public List<BentenTmSearchResult> fuzzyMatch(String string) {
		synchronized (this) {
			if (tm == null) {
				log("fuzzyMatch:TMエンジンがロードされていません。");
				return new ArrayList<BentenTmSearchResult>();
			}
			final IPreferenceStore store = BentenPreference.getStore();
			final boolean ignoreWhitespace = store.getBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name());
			if (ignoreWhitespace) {
				string = Strings.removeWhitespace(string);
			}
			return tm.fuzzySearch(string);
		}
	}

	/**
	 * このプラグインが保持している TM エンジンを取得します。
	 *
	 * @return TM エンジン
	 */
	public BentenTmEngine getTmEngine() {
		return tm;
	}

	/**
	 * TM エンジンのインスタンスを新規に作成します。
	 *
	 * 所定のルールに従って拡張ポイントを検索して TM エンジンのインスタンスを作成します。
	 *
	 * @return TM エンジンのインスタンス。
	 */
	public BentenTmEngine newTmEngine() {
		final Map<String, BentenTmEngine> tmMap = new HashMap<String, BentenTmEngine>();
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.tm.engine");
		if (point == null) {
			return null;
		}

		// このシステムに登録されている、すべての翻訳メモリー・エンジンの一覧を取得。
		// 取得された翻訳メモリー･エンジンの名前を記憶。
		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name");
					final Object helloObj = cfgElem.createExecutableExtension("class");
					if (helloObj instanceof BentenTmEngine) {
						tmMap.put(name, (BentenTmEngine) helloObj);
					}
				} catch (final CoreException e) {
					log(e);
				}
			}
		}

		// 「設定」から翻訳メモリー･エンジンの名称を取得。
		// 該当する翻訳メモリー・エンジンの翻訳メモリーをロードする。
		final IPreferenceStore store = BentenPreference.getStore();
		final String tmName = store.getString(Preference.TM_NAME.name());
		final BentenTmEngine tm = tmMap.get(tmName);
		return tm;
	}
}
