/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui.actions;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import benten.cat.tm.core.BentenTmSearchResult;
import benten.cat.tm.ui.CatTmUiPlugin;
import benten.cat.tm.ui.views.TranslationView;

public class ShowTranslationAction implements IEditorActionDelegate {

	private static final Map<String, List<BentenTmSearchResult>> cache = new WeakHashMap<String, List<BentenTmSearchResult>>();
	private IEditorPart targetEditor;
	private ISelection selection;

	public void selectionChanged(IAction action, ISelection selection) {
		this.selection = selection;
	}

	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		this.targetEditor = targetEditor;
	}

	public void run(IAction action) {

		final String text = ((ITextSelection) selection).getText();
		final TableViewer viewer = showView().getViewer();

		final List<BentenTmSearchResult> list = cache.get(text);
		if (list != null) {
			viewer.setInput(list);
			return;
		}

		final Job job = new Job("翻訳中...") {
			@Override
			protected IStatus run(final IProgressMonitor monitor) {
				monitor.beginTask(null, IProgressMonitor.UNKNOWN);
				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				final List<BentenTmSearchResult> list = CatTmUiPlugin.getDefault().fuzzyMatch(text);
				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						viewer.setInput(list);
						cache.put(text, new LinkedList<BentenTmSearchResult>(list));
					}
				});
				return Status.OK_STATUS;
			}
		};
		job.setUser(true);
		job.schedule();
	}

	private TranslationView showView() {
		try {
			return (TranslationView) targetEditor.getSite().getPage().showView(TranslationView.class.getName());
		} catch (final PartInitException e) {
			CatTmUiPlugin.getDefault().log(e);
			throw new IllegalStateException(e);
		}
	}
}
