/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui.preference;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;
import benten.ui.views.UiToolkit;

/**
 * TM 設定ページです。
 */
public class CatTmUiPreferencePage extends PreferencePage implements IWorkbenchPreferencePage {

	private final UiToolkit toolkit = new UiToolkit();
	private Text tmText;
	private Button ignoreWhitespaceTmReferenceCheck;
	private Button ignoreWhitespaceTmxExtractCheck;

	public CatTmUiPreferencePage() {
	}

	public CatTmUiPreferencePage(String title) {
		super(title);
	}

	public CatTmUiPreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		final Composite composite = toolkit.createComposite(parent, 1);

		final Composite c = toolkit.createComposite(composite, 2);
		c.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		toolkit.createLabel(c, "翻訳メモリー:");

		tmText = new Text(c, SWT.SINGLE | SWT.BORDER);
		tmText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		final IPreferenceStore sotre = BentenPreference.getStore();
		tmText.setText(sotre.getString(Preference.TM_NAME.name()));

		toolkit.createLabel(composite, "");

		final Group g = new Group(composite, SWT.BOLD);
		g.setLayout(toolkit.createGridLayout(1, 5));
		g.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		g.setText("空白や改行の扱い");
		final IPreferenceStore store = BentenPreference.getStore();

		ignoreWhitespaceTmReferenceCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmReferenceCheck.setText("TM 参照時に空白や改行を無視(&R)");
		ignoreWhitespaceTmReferenceCheck.setSelection(store
				.getBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name()));

		ignoreWhitespaceTmxExtractCheck = new Button(g, SWT.CHECK);
		ignoreWhitespaceTmxExtractCheck.setText("TMX 抽出時に空白や改行を無視(&E)");
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getBoolean(Preference.IGNORE_WHITESPACE_TMX_EXTRACT.name()));

		return composite;
	}

	public void init(IWorkbench workbench) {
	}

	@Override
	public boolean performOk() {
		final IPreferenceStore store = BentenPreference.getStore();
		store.setValue(Preference.TM_NAME.name(), tmText.getText());
		store.setValue(Preference.IGNORE_WHITESPACE_TM_REFERENCE.name(), ignoreWhitespaceTmReferenceCheck
				.getSelection());
		store.setValue(Preference.IGNORE_WHITESPACE_TMX_EXTRACT.name(), ignoreWhitespaceTmxExtractCheck.getSelection());
		return true;
	}

	@Override
	protected void performDefaults() {
		final IPreferenceStore store = BentenPreference.getStore();
		tmText.setText(store.getDefaultString(Preference.TM_NAME.name()));
		ignoreWhitespaceTmReferenceCheck.setSelection(store.getDefaultBoolean(Preference.IGNORE_WHITESPACE_TM_REFERENCE
				.name()));
		ignoreWhitespaceTmxExtractCheck.setSelection(store.getDefaultBoolean(Preference.IGNORE_WHITESPACE_TMX_EXTRACT
				.name()));
	}
}
