/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.tm.ui.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;

import benten.cat.tm.core.BentenTmSearchResult;
import benten.cat.tm.ui.CatTmUiPlugin;
import benten.cat.tm.ui.actions.CopyTranslationSourceAction;
import benten.cat.tm.ui.actions.CopyTranslationTargetAction;
import benten.ui.views.CopySupportTableView;

public class TranslationView extends CopySupportTableView {

	private TableColumn tmName;
	private TableColumn source;
	private TableColumn translation;
	private TableColumn matchQuality;

	private Action copySourceAction;
	private Action copyTargetAction;

	class ViewLabelProvider extends LabelProvider implements ITableLabelProvider {

		public String getColumnText(Object obj, int index) {
			BentenTmSearchResult entry = (BentenTmSearchResult) obj;
			switch (index) {
			case 0:
				return entry.getOrigin();
			case 1:
				return entry.getMatchQuality();
			case 2:
				return entry.getSource();
			case 3:
				return entry.getTarget();
			}
			return getText(obj);
		}

		public Image getColumnImage(Object obj, int index) {
			return null;
		}
	}

	class NameSorter extends ViewerSorter {
	}

	/**
	 * コンストラクター。
	 */
	public TranslationView() {
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		viewer = new TableViewer(parent, SWT.MULTI | SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		tmName = new TableColumn(table, SWT.RIGHT);
		tmName.setText("TM");
		tmName.setWidth(80);

		matchQuality = new TableColumn(table, SWT.RIGHT);
		matchQuality.setText("品質");
		matchQuality.setWidth(50);

		source = new TableColumn(table, SWT.LEFT);
		source.setText("ソース");
		source.setWidth(200);

		translation = new TableColumn(table, SWT.LEFT);
		translation.setText("ターゲット");
		translation.setWidth(200);

		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(new ViewLabelProvider());
		//		viewer.setSorter(new NameSorter());
		viewer.setInput(getItems());

		makeActions();
		hookContextMenu();
	}

	private List<BentenTmSearchResult> getItems() {
		return new ArrayList<BentenTmSearchResult>();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				TranslationView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void fillContextMenu(IMenuManager manager) {
		boolean isEmpty = viewer.getSelection().isEmpty();
		copySourceAction.setEnabled(!isEmpty);
		manager.add(copySourceAction);
		copyTargetAction.setEnabled(!isEmpty);
		manager.add(copyTargetAction);

		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	public void showTranslation(String string) {
		CatTmUiPlugin plugin = CatTmUiPlugin.getDefault();
		viewer.setInput(plugin.fuzzyMatch(string));
	}

	private void makeActions() {
		copySourceAction = new CopyTranslationSourceAction(this, "ソースのコピー");
		copyTargetAction = new CopyTranslationTargetAction(this, "ターゲットのコピー");
	}
}
